#include <QtCore/QDebug>

#include "MediaControler.h"

#define VIDEO_SRC "videotestsrc"
#define VIDEO_SINK "ximagesink"

MediaControler::MediaControler( QObject *parent )
                : QObject( parent )
{
    gst_init();
    //gst_init_2();
}

MediaControler::~MediaControler()
{
    if( bus )
    {
        g_object_unref( bus );
        bus = NULL;
    }

    gst_element_set_state( pipeline, GST_STATE_NULL );
    if( pipeline )
    {
        g_object_unref( pipeline );
        pipeline = NULL;
    }

    if( audioInput )
    {
        g_object_unref( audioInput );
        audioInput = NULL;
    }

    if( audioOutputBin )
    {
        g_object_unref( audioOutputBin );
        audioOutputBin = NULL;
    }
    //GstElement *videoInputBin;
    //GstElement *videoTee;
}

void MediaControler::gst_init()
{
    //生成管道
    pipeline = gst_pipeline_new( "mypipeline" );
    //从管道得到dbus
    bus = gst_pipeline_get_bus( GST_PIPELINE(pipeline) );
    gst_bus_add_watch( bus, (GstBusFunc)&gst_bus_watch_cb, this );

    //生成自动音频输入
    audioInput = gst_element_factory_make( "autoaudiosrc", "audio_input" );
    //引用计数加1
    gst_object_ref( audioInput );
    //去除新生成对象的floating flag，引用计数减1
    gst_object_sink( audioInput );

    //new一个音频输出bin
    audioOutputBin = gst_bin_new( "audio_output_bin" );
    //
    GstElement *resample = gst_element_factory_make( "audioresample", "audio_resample" );
    //生成一个自动音频输出
    GstElement *audioSink = gst_element_factory_make( "autoaudiosink", "audio_sink" );
    //把 resample，audioSink添加到bin里面
    gst_bin_add_many( GST_BIN(audioOutputBin), resample, audioSink, NULL );
    //链接 resample, audioSink
    gst_element_link_many( resample, audioSink, NULL );
    //得到 resample的sink pad
    GstPad *sink = gst_element_get_static_pad( resample, "sink" );
    // 从sink生成一个ghost pad
    GstPad *ghost = gst_ghost_pad_new( "sink", sink );
    // 把ghost pad添加到自动音频输出bin
    gst_element_add_pad( GST_ELEMENT( audioOutputBin ), ghost );
    gst_object_unref( G_OBJECT( sink ) );
    gst_object_ref( audioOutputBin );
    gst_object_sink( audioOutputBin );

    videoInputBin = gst_bin_new( "video_input_bin" );
    GstElement *scale = gst_element_factory_make( "videoscale", "video_scale" );
    GstElement *rate = gst_element_factory_make( "videorate", "video_rate" );
    GstElement *colorspace = gst_element_factory_make( "ffmpegcolorespace", "cs" );
    GstElement *capsfilter = gst_element_factory_make( "capsfilter", "csp_filter" );
    GstCaps *caps = gst_caps_new_simple( "video/x-raw-rgb",
                                        "width", G_TYPE_INT, 320,
                                        "height", G_TYPE_INT, 240,
                                        //"bpp", G_TYPE_INT, 24,
                                        //"depth", G_TYPE_INT, 24,
                                        NULL );
    g_object_set( G_OBJECT( capsfilter ), "caps", caps, NULL );
    //GstElement *videoSrc = gst_element_factory_make( "autovideosrc", "video_src" );
    GstElement *videoSrc = gst_element_factory_make( "videotestsrc", "video_src" );
    gst_bin_add_many( GST_BIN( videoInputBin ), videoSrc, scale, rate, colorspace, capsfilter, NULL );
    gst_element_link_many( videoSrc, scale, rate, colorspace, capsfilter, NULL );
    GstPad *src = gst_element_get_static_pad( capsfilter, "src" );
    ghost = gst_ghost_pad_new( "src", src );
    gst_element_add_pad( GST_ELEMENT( videoInputBin ), ghost );
    gst_object_unref( src );
    gst_object_ref( videoInputBin );
    gst_object_sink( videoInputBin );

    videoTee = gst_element_factory_make( "tee", "tee" );
    gst_object_ref( videoTee );
    gst_object_sink( videoTee );

//    GstElement screen_sink = gst_element_factory_make( "fakesink", "screen_sink" );
//    gst_object_ref( screen_sink );
//    gst_object_sink( screen_sink );

    videoPreview = new VideoWidget( bus );
    GstElement *videoPreviewElement = videoPreview->element();

    gst_bin_add_many( GST_BIN( pipeline ), videoInputBin, videoTee, videoPreviewElement, NULL );
    gst_element_link_many( videoInputBin, videoTee, videoPreviewElement, NULL );

    videoOutput = new VideoWidget( bus );

    gst_element_set_state( pipeline , GST_STATE_PLAYING );
    status = StatusConnecting;

    //videoPreview->exposeed();
}

void MediaControler::gst_init_2()
{
/*    GstElement *camera_src, *screen_sink, *image_sink;
    GstElement *screen_queue, *image_queue;
    GstElement *csp_filter, *image_filter, *tee;
    GstCaps *caps;

    //生成管道
    pipeline = gst_pipeline_new( "mypipeline" );
    //从管道得到dbus
    bus = gst_pipeline_get_bus( GST_PIPELINE(pipeline) );
    gst_bus_add_watch( bus, (GstBusFunc)&gst_bus_watch_cb, this );
    gst_object_unref(GST_OBJECT(bus));


        camera_src = gst_element_factory_make(VIDEO_SRC, "camera_src");

        csp_filter = gst_element_factory_make("ffmpegcolorspace", "csp_filter");

        tee = gst_element_factory_make("tee", "tee");

        screen_queue = gst_element_factory_make("queue", "screen_queue");

        screen_sink = gst_element_factory_make(VIDEO_SINK, "screen_sink");

        image_queue = gst_element_factory_make("queue", "image_queue");

        image_filter = gst_element_factory_make("ffmpegcolorspace", "image_filter");

        image_sink = gst_element_factory_make("fakesink", "image_sink");

        if(!(pipeline && camera_src && screen_sink && csp_filter && screen_queue
                && image_queue && image_filter && image_sink))
        {
                g_critical("Couldn't create pipeline elements");
                return FALSE;
        }

        gst_bin_add_many(GST_BIN(pipeline), camera_src, csp_filter,
                        tee, screen_queue, screen_sink, image_queue,
                        image_filter, image_sink, NULL);

        caps = gst_caps_new_simple("video/x-raw-rgb",
                        "width", G_TYPE_INT, 640,
                        "height", G_TYPE_INT, 480,
                        NULL);


        if(!gst_element_link_filtered(camera_src, csp_filter, caps))
        {
                return FALSE;
        }
        gst_caps_unref(caps);

        if(!gst_element_link_many(csp_filter, tee, screen_queue, screen_sink, NULL))
        {
                return FALSE;
        }

        gst_element_set_state(pipeline, GST_STATE_PLAYING);
*/
	videoPreview = new VideoWidget( bus );

    videoOutput = new VideoWidget( bus );

}

gboolean MediaControler::gst_bus_watch_cb( GstBus *bus, GstMessage *message, MediaControler *self )
{
    //qDebug() << "MediaControler::gst_bus_watch_cb Entry";
    return true;
}

VideoWidget *MediaControler::videoPreviewWidget() const
{
    return videoPreview;
}

VideoWidget *MediaControler::videoOutputWidget() const
{
    return videoOutput;
}
