#include <QApplication>
#include <QtCore/QDebug>

#include "VideoWidget.h"

#include <gst/interfaces/xoverlay.h>

extern void qt_x11_set_global_double_buffer(bool);

struct VideoWidget::Private
{
    Private( VideoWidget *parent, GstBus *bus );
    ~Private();

    static void onSyncMessage( GstBus *bus, GstMessage *message, Private *self );

    VideoWidget *parent;
    GstBus *bus;
    GstElement *sink;
    GstElement *overlay;
};

VideoWidget::Private::Private( VideoWidget *parent, GstBus *bus )
            : parent( parent ),
            bus( (GstBus*)gst_object_ref( bus ) ),
            overlay( NULL )
{
    //sink = gst_element_factory_make( "autovideosink", NULL );
    sink = gst_element_factory_make( "ximagesink", "screen_sink" );
    gst_object_ref( sink );
    gst_object_sink( sink );

    gst_bus_enable_sync_message_emission( bus );
    g_signal_connect( bus, "sync-message", G_CALLBACK( &VideoWidget::Private::onSyncMessage ), this );

    overlay = sink;
}

VideoWidget::Private::~Private()
{
    gst_object_unref( bus );
    gst_object_unref( sink );
}

void VideoWidget::Private::onSyncMessage( GstBus *bus, GstMessage *message, Private *self )
{
    //qDebug() << "VideoWidget::Private::onSyncMessage: Entry";
    if( GST_MESSAGE_TYPE( message ) != GST_MESSAGE_ELEMENT )
    {
        //qDebug() << "VideoWidget::Private::onSyncMessage: not gst_message_element";
        return;
    }
    if( GST_MESSAGE_SRC( message ) != (GstObject*)self->overlay )
    {
        //qDebug() << "VideoWidget::Private::onSyncMessage: not overlay";
        return;
    }

    const GstStructure *s = gst_message_get_structure( message );
    if( gst_structure_has_name( s, "prepare-xwindow-id" ) && self->overlay )
    {
        qDebug() << "VideoWidget::Private::onSyncMessage: invokeMethod( setOverlay )";
        QMetaObject::invokeMethod( self->parent, "setOverlay", Qt::QueuedConnection );
    }
    //qDebug() << "VideoWidget::Private::onSyncMessage: Exit";
}

VideoWidget::VideoWidget( GstBus *bus, QWidget *parent)
            : QWidget( parent ),
            mPriv( new Private( this, bus ) )
{
/*    qt_x11_set_global_double_buffer( false );

    QPalette palette;
    palette.setColor( QPalette::Background, Qt::black );
    setPalette( palette );
    setAutoFillBackground( true );
*/
	QCoreApplication::setAttribute( Qt::AA_NativeWindows );
}

VideoWidget::~VideoWidget()
{
    delete mPriv;
}

GstElement *VideoWidget::element() const
{
    return mPriv->sink;
}

void VideoWidget::setOverlay()
{
    qDebug() << "VideoWidget::setOverlay Entry";
    //if( mPriv->overlay && GST_IS_X_OVERLAY( mPriv->overlay ) )
    {
        qDebug() << "VideoWidget::setOverlay gst_x_overlay_set_xwindow_id";
        WId windowId = winId();
        QApplication::syncX();
        qDebug() << "windowId = " << windowId;
        gst_x_overlay_set_xwindow_id( GST_X_OVERLAY( mPriv->overlay ), windowId );
    }
    windowExposed();
    qDebug() << "VideoWidget::setOverlay Exit";
}

void VideoWidget::windowExposed()
{
    qDebug() << "VideoWidget::windowExposed Entry";
    QApplication::syncX();
    //if( mPriv->overlay && GST_IS_X_OVERLAY( mPriv->overlay ) )
    {
        qDebug() << "VideoWidget::windowExposed gst_x_overlay_expose";
        gst_x_overlay_expose( GST_X_OVERLAY( mPriv->overlay ) );
    }
    qDebug() << "VideoWidget::windowExposed Exit";
}

void VideoWidget::exposeed()
{
    qDebug() << "VideoWidget::exposeed Entry";
    setOverlay();
    qDebug() << "VideoWidget::exposeed Exit";
}

void VideoWidget::showEvent( QShowEvent* event )
{
	qDebug() << "VideoWidget::showEvent Entry";
	exposeed();
	qDebug() << "VideoWidget::showEvent Exit";
}
